<document>
:@* Appendix: The slides format

In this technical appendix, a simplified description will be
given of the slides format.
Slides are fragments of a document that may be presented to an audience.
Our approach allows for displaying slides in either
dynamic HTML or VRML.
Slides are encoded using <url href=http://www.xml.org>XML</url>.
%%The conversion to these formats takes place using XSLT,
%%the XML transformation language.

<slide id=xml-slide-example>
/// <a href=@xml-xml-slide.html>slides in XML</a>
<include file=plain>
<small>
<display>
&lt;document&gt;
...
&lt;slide id="1"&gt;
&lt;text&gt;
&lt;line&gt;What about the slide format?&lt;/line&gt;
&lt;break/&gt;
&lt;line&gt;yeh, what about it"?&lt;/line&gt;
&lt;/text&gt;
&lt;vrml&gt;Sphere { radius 0.5 }&lt;/vrml&gt;
&lt;/slide&gt;
...
&lt;slide id="2"&gt;
&lt;vrml&gt;Sphere { radius 0.5 }&lt;/vrml&gt;
&lt;/slide&gt;
...
&lt;/document&gt;
</display>
</small>
<navigate>
</slide>
\noindent
The first slide contains some text and a 3D object.
The second slide contains only a 3D object.
Inbetween the slides there may be arbitrary text.
The slides are converted to VRML using XSLT,
the XML transformation language.

:@+ VRML PROTOs
To support slides in VRML a small collection 
of PROTO definitions is used.
%%On the VRML-side,
%%For displaying text in VRML
%%we need only a small collection of PROTOs
%%<footnote>
%%The PROTOs were initially developed by
%%Alex van Ballegooij, who also did the majority
%%of the coding of an extended collection of PROTOs.</footnote>

%%<c><p align=right><a href=@vr-example-slide.html>example</a></p></c>
%%<c><p align=right><a href=proto.html>proto(s)</a></p></c>

<slide id=web3d-sl-protos>
<c>
/// <a href=proto.html><em>protos</em></a>
</c>
%%<small>
<p>
<ul bullet=red> 
<li> <a href=@vr-proto-slideset.html><em>slideset</em></a> -- container for slides</li>
<li> <a href=@vr-proto-slide.html><em>slide</em></a> -- container for text and objects</li>
<li> <a href=@vr-proto-text.html><em>slide</em></a> -- container for lines of text</li>
<li> <a href=@vr-proto-line.html><em>line</em></a> -- container for text</li>
<li> <a href=@vr-proto-break.html><em>break</em></a> -- empty text</li>
</ul>
%%</small>
<navigate/>
</slide>
\noindent
The slides contained in a document constitute a slide set.
A slide set is a collection of slides that may contain lines of text
and possibly 3D objects.
For displaying 3D objects in a slide
we need no specific PROTO.
<lp>
The <em>slide</em> PROTO defines an interface which may be used
to perform spatial transformations on the slide,
like translation, rotation and scaling.
The interface also includes a field to declare the
content of the slide, that is text or (arbitrary) 3D objects.
<lp>
The <em>slideset</em> contains a collection of slides, and allows
for proceeding to the next slide.
A <em>text</em> may contain a sequence of lines and breaks.
It supports a simple layout algorithm.
%%<lp>
%%The interface of the <em>slideset</em> PROTO,
%%which corresponds to the <em>document</em> tag,
%%allows for declaring which slides belong
%%to the set of slides.
%%In addition, the <em>slideset</em> PROTO interface also contains
%%a so-called <em>eventIn</em> named <em>next</em>
%%to proceed to the next slide.
%%<lp>
%%To select between the different slides
%%a <em>Switch</em> node is used,
%%which is controlled by a <em>Script</em>.


:@+ annotated slides
Slides may be annotated with <em>dialogs</em>,
as described in section 5.
The annotation is compiled to DLP code,
which is activated whenever the slide (or level within a slide)
to which the annotation belongs is displayed.
<lp>
To intercept the occurrence of a particular event,
such as the display of a slide, we use an <em>observer</em>
object which is specified as in the code fragment below.
%%<lp>
<lp>
<slide color=blue a=0 id=pl-dlp-observer>
%%<c>
/// <em>observer</em>
%%</c>
<small>
<display text=white font=tt>
:- object observer : [actions].
var slide = anonymous, level = 0, projector = nil.

observer(X) :- 
   projector := X,
   repeat,
     accept( id, level, update, touched),
   fail.

id(V) :-  slide := V.
level(V) :- level := V.
touched(V) :- projector$<-$touched(V).
update(V) :- act(V,slide,level).
:- end_object observer.
</display>
</small>
</slide>
\noindent
The observer object
has knowledge of, that is inherits from, an object
that contains particular actions.
<lp>
%%When a new instance of an <em>observer</em>
%%is created,
%%the non-logical variable <em>projector</em> is set by
%%the constructor, 
%%and the instance enters a repeat loop to accept
%%any of the incoming events for respectively <em>id</em>,
%%<em>level</em>, <em>update</em> and <em>touched</em>.
%%Each event has a value, that is available as a parameter
%%when the corresponding method is called on the acceptance
%%of the event.
%%To receive events, the <em>observer</em> object
%%must be installed as the listener for these particular events.
<lp>
As indicated before, events come from the 3D scene.
For example, the <em>touched</em> event results from mouse clicks
on a particular object in the scene.
On accepting an event, the corresponding method or clause
is activated, resulting in either changing the value
of a non-logical instance variable,
invoking a method,
or delegating the call to another object.
%%<lp>
%%An observer of this kind is used 
%%to start a dialog on the occurrence of a particular slide.

</document>
