<document>
:@* Appendix: The slides format

In this technical appendix, a simplified description will be
given of the slides format.
Slides are fragments of a document that may be presented to an audience.
Our approach allows for displaying slides in either
dynamic HTML or VRML.
Slides are encoded using <url href=http://www.xml.org>XML</url>.
The conversion to these formats takes place using XSLT,
the XML transformation language.
<lp>
The slides contained in a document constitute a slide set.
A slide set is a collection of slides that may contain lines of text
and possibly 3D objects.
Let's look at the XML-encoding of some example
slides.

<slide id=xml-slide-example>
/// <a href=@xml-xml-slide.html>slides in XML</a>
<small>
<display>
&lt;document&gt;
...
&lt;slide id="1"&gt;
&lt;text&gt;
&lt;line&gt;What about the slide format?&lt;/line&gt;
&lt;break/&gt;
&lt;line&gt;yeh, what about it"?&lt;/line&gt;
&lt;/text&gt;
&lt;vrml&gt;Sphere { radius 0.5 }&lt;/vrml&gt;
&lt;/slide&gt;
...
&lt;slide id="2"&gt;
&lt;vrml&gt;Sphere { radius 0.5 }&lt;/vrml&gt;
&lt;/slide&gt;
...
&lt;/document&gt;
</display>
</small>
<navigate>
</slide>
\noindent
The first slide contains some text and a 3D object.
The second slide contains only a 3D object.
Inbetween the slides there may be arbitrary text.
The slides are converted to VRML using XSLT.
To support slides in VRML a small collection 
of PROTO definitions is used, as described below.
<lp>
%%Before looking at the stylesheet used for the conversion
%%to VRML, let me briefly say something about XSLT.
The XSLT transformation language is a declarative
language.
It allows for processing an XML-encoded text by templates
matching particular tags.
In addition, the values of attributes of tags
may be used when generating output.
<lp>
The first part of our XSLT stylesheet for converting
slides to VRML looks as follows.
<slide id=xml-stlyle-slide>
/// <a href=@xsl-xsl-slide.html>XSLT <em>stylesheet</em></a>
<small>
<display>
&lt;?xml version="1.0"?&gt;
&lt;xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"&gt;
&lt;xsl:output method="text"/&gt;
</display></small>
</slide>
\noindent
Apart from the obligatory declaration
that the stylesheet itself is written in XML,
there is also the indication that the file is a stylesheet.
%%written according to the rules and conventions that can be found in
%%a file dating from 1999, as given in the url.
Since we do want to generate VRML (and not XML or HTML),
we need to indicate that our output method is <em>text</em>,
to avoid having an XML header at the start of the output.
<lp>
%%Now we are ready to define our first template.
The template for <em>document</em>,
that will result in the declaration of a <em>slideset</em>,
looks as follows.

<slide id=xml-style-slideset>
/// <em>document</em>
<small>
<display>
&lt;xsl:template match="/document"&gt;

<em>... load (extern) proto(s)</em>

DEF slides slideset {
slides [
&lt;xsl:apply-templates/&gt;
] # slides
}

<em>... include user interface</em>

&lt;/xsl:template&gt;
</display>
</small>
</slide>
\noindent
Everything that is not part of a tag containing
the <em>xsl</em> prefix is literally copied
to output.
In this fragment, the full 
PROTO declarations have not been included 
nor the code for the user interface needed
to traverse the slides.
In the middle of the fragment we see the <em>xsl</em> tag
<em>apply-templates</em>.
This results in further processing the content that is
contained between the <em>document</em> begin and end tag,
using the template definitions given below. 
<lp>
The template for the <em>slide</em> tag is simple.

<slide id=xml-style-text>
/// <em>slide</em>
<small>
<display>
&lt;xsl:template match="*/slide"&gt;
slide { children [
&lt;xsl:apply-templates/&gt;
] }
&lt;/xsl:template&gt;
</display>
</small>
</slide>
\noindent
<lp>
The templates for the <em>text</em>, <em>line</em> and <em>break</em> are equally simple.
They just result in creating an instance of the appropriate
prototypes (see below) with the appropriate content.
The template for the VRML tag is even more simple,
it literally copies the content  inbetween the <em>vrml</em> 
begin and end tag.

:@+ VRML PROTOs
On the VRML-side, we need just a collection of PROTOs
for displaying text in VRML.<footnote>
The PROTOs were initially developed by
Alex van Ballegooij, who also did the majority
of the coding of an extended collection of PROTOs.</footnote>

%%<c><p align=right><a href=@vr-example-slide.html>example</a></p></c>
%%<c><p align=right><a href=proto.html>proto(s)</a></p></c>

<slide id=web3d-sl-protos>
/// <a href=proto.html><em>protos</em></a>
<small>
<ul> 
<li> <a href=@vr-proto-slideset.html><em>slideset</em></a> -- container for slides</li>
<li> <a href=@vr-proto-slide.html><em>slide</em></a> -- container for text and objects</li>
<li> <a href=@vr-proto-text.html><em>slide</em></a> -- container for lines of text</li>
<li> <a href=@vr-proto-line.html><em>line</em></a> -- container for text</li>
<li> <a href=@vr-proto-break.html><em>break</em></a> -- empty text</li>
</ul>
</small>
<navigate/>
</slide>
\noindent
Note that for displaying 3D objects in a slide,
we need no specific PROTO.
<lp>
Before looking at the PROTO for a set of slides,
let's look at the <em>slide</em> PROTO.
It is surprisingly simple.

<slide id=web3d-proto-slide>
/// <em>slide</em> 
<small>
<display>
PROTO slide [
    exposedField SFVec3f    translation 0 0 15 
    exposedField SFRotation rotation    0 1 0 0
    exposedField SFVec3f    scale       1 1 1
    exposedField MFNode     children []
] {
    Transform {
        children    IS children
        translation IS translation
        rotation    IS rotation
        scale       IS scale
    }
}
</display>
</small>
</slide>
\noindent
The <em>slide</em> PROTO defines an interface which may be used
to perform spatial transformations on the slide,
like translation, rotation and scaling.
The interface also includes a field to declare the
content of the slide, that is text or (arbitrary) 3D objects.
<lp>
The interface of the <em>slideset</em> PROTO,
which corresponds to the <em>document</em> tag,
allows for declaring which slides belong
to the set of slides.

<slide id=web3d-slideset-proto>
/// <em>slideset</em>
<small>
<display>
PROTO slideset [
    exposedField SFInt32    visible 0
    exposedField MFNode     slides []
    eventIn SFInt32         next
] {
    DEF select Switch {
        choice      IS slides
        whichChoice IS visible
    }

    Script {
    ... 
    }

} 
</display>
</small>
<navigate/>
</slide>
\noindent
Apart from the <em>visible</em> field,
which may be used to start a presentation
with another slide than the first one
(zero being the first index in the array of slides),
the <em>slideset</em> PROTO interface also contains
a so-called <em>eventIn</em> named <em>next</em>
to proceed to the next slide.
<lp>
To select between the different slides
a <em>Switch</em> node is used,
which is controlled by a <em>Script</em>.
%%The code of the script is given below.
%%
%%<slide id=web3d-sl-s-script>
%%/// <em>script</em>
%%<small>
%%<display>
%%    Script {
%%    directOutput TRUE
%%    eventIn SFInt32 next IS next
%%    field SFInt32 slide IS visible
%%    field SFNode select USE select
%%    field MFNode slides []
%%    url "javascript:
%%    function next(value) {
%%    slides = select.choice;
%%    Browser.print('=' + slide + ' ' + slides.length);
%%    if (slide == (slides.length-1)) slide = 0;
%%    else slide += 1;
%%    select.whichChoice = slide;
%%    }"
%%    } 
%%</display>
%%</small>
%%<navigate/>
%%</slide>
%%\noindent
%%In the interface of the script, we see both the use of <em>IS</em>
%%and <em>USE</em> to connect the (local) script fields
%%to the scenegraph.
The <em>Script</em> contains a 
function <em>next</em>, that implements
the corresponding event. This function simply
traverses through the slides,
one step at a time, by assigning a value to
the <em>whichChoice</em> field of the <em>Switch</em>.


:@+ annotated slides
Slides may be annotated with <em>dialogs</em>,
as described in section 5.
The annotation is compiled to DLP code,
which is activated whenever the slide (or level within a slide)
to which the annotation belongs is displayed.
<lp>
To intercept the occurrence of a particular event,
such as the display of a slide, we use an <em>observer</em>
object which is specified as in the code fragment below.
%%<lp>
<lp>
<slide color=blue a=0 id=pl-dlp-observer>
%%<c>
/// <em>observer</em>
%%</c>
<small>
<display text=white font=tt>
:- object observer : [actions].
var slide = anonymous, level = 0, projector = nil.

observer(X) :- 
   projector := X,
   repeat,
     accept( id, level, update, touched),
   fail.

id(V) :-  slide := V.
level(V) :- level := V.
touched(V) :- projector$<-$touched(V).
update(V) :- act(V,slide,level).
:- end_object observer.
</display>
</small>
</slide>
\noindent
The observer object
has knowledge of, that is inherits from, an object
that contains particular actions.
<lp>
When a new instance of an <em>observer</em>
is created,
the non-logical variable <em>projector</em> is set by
the constructor, 
and the instance enters a repeat loop to accept
any of the incoming events for respectively <em>id</em>,
<em>level</em>, <em>update</em> and <em>touched</em>.
Each event has a value, that is available as a parameter
when the corresponding method is called on the acceptance
of the event.
To receive events, the <em>observer</em> object
must be installed as the listener for these particular events.
<lp>
As indicated before, events come from the 3D scene.
For example, the <em>touched</em> event results from mouse clicks
on a particular object in the scene.
On accepting an event, the corresponding method or clause
is activated, resulting in either changing the value
of a non-logical instance variable,
invoking a method,
or delegating the call to another object.
%%<lp>
%%An observer of this kind is used 
%%to start a dialog on the occurrence of a particular slide.

</document>
