<document type=architecture>
%%with-pdf no-dlp no-wrl
<c><p align=right><a href=@text-paper-architecture.pdf>PDF</a></p></c>
<so abstract-architecture>
<include file=@biblio/keys>

@* Introduction

A variety of applications may benefit from deploying
embodied conversational agents, either in the form
of animated humanoid avatars or, more simply, as a 'talking head'.
An interesting example is provided by <em>Signing Avatars</em>,
a system
that allows for translating arbitrary Web pages in both spoken
language and sign language for the deaf, presented by animated
humanoid avatars.
Here the use of animated avatars is essential to communicate
with a particular group of users, using the sign language
for the deaf.
<lp>
Other applications of embodied conversational agents
include e-commerce and social marketing,
although in these cases it may not always be evident that
animated avatars or faces actually do provide added value.
<lp>
Another usage of embodied conversational agents
may be observed in virtual environments
such as Active Worlds, blaxxun Community or Adobe Atmosphere.
Despite the rich literary background of such environments,
including Snow Crash,
the functionality of such agents is usually rather shallow,
due to the poor repertoire of gestures and movements
on the one hand and the restricted computational model
underlying these agents on the other hand. 
In effect, the definition of agent avatars in virtual
environments generally relies on a proprietary
scripting language which, as in the case
of <em>blaxxun</em> Agents,offers only limited
pattern matching and a fixed repertoire of built-in actions.
<lp>
In contrast, the scripting language for <em>Signing Avatars</em>
is based on the H-Anim standard and allows
for a precise defintion of a complex repertoire
of gestures, as exemplified by the sign language for the deaf.
Nevertheless, also this scripting language is of a
proprietary nature and does not allow for higher-order
abstractions of semantically meaningful behavior.
<lp>
In this paper, we propose an architecture and software platform
for embodied conversational agents that provides
the computational facilities for defining semantically meaningful
behaviors and that allows for a rich presentational
environment,
in particular 3D virtual environments that may include
streaming video, text and speech.



:@ structure 
The structure of this papers is as follows.
In section 2, we
will outline the requirements
for a rich media software platform supporting embodied conversational
agents.
Then, in section 3, we
will propose an architecture based on the 
distributed logic programming language DLP and X3D.
In section 4, 
three case studies will be described,
illustrating the potential of the DLP+X3D platform.
In section 5, we will discuss related work and
in section 6 we will
review the evaluation criteria that pertain to the
deployment of embodied conversational agents
in rich media 3D environments.
Finally, in section 7, we will give some conclusions
and indications for future work.

@* Requirements

There is a wide range of presentation platform for embodied
conversational agents.
On one end of the spectrum we have digitized video,
as for example in the early versions of the Ananova newsreader,
[_Ananova].
On the other end of the spectrum we have rich media real-time 3D
platforms, as in the aforementioned virtual environments.
<lp>
For both type of systems, advanced authoring tools 
may be used to create content.
In addition, however, for the latter type of
systems also declarative means of modeling may be used
as well as a programmatic interface to create dynamic content.
<lp>
Based on our experiences, 
which are reported in section 4,
we have a clear preference for web-based real-time 3D environments
with a strong programmatic interface.
Our requirements with respect to a platform supporting such environments
may be summarized as follows.

<slide id=ar-reqs>
<c>
/// <em>programming platform</em>  
</c><p>
<small>
<ul bullet=red size=1.2>
<li><em>declarative language</em> -- for agent support</li>
%%<li><em>programmability</em> -- X3D/VRML EAI support </li>
<li><em>multiple threads of control</em> -- for multiple shared objects</li>
<li><em>distributed communication</em> -- networking capabilities (TCP/IP)</li>
</ul>
</small>
%%<effect bg=-1 type=style r_bulb=0 l_bulb=0/>
<phrase level=0 right="Is this where~nDLP~ncomes in?">
<phrase level=0 left="I think so!"/>
<navigate/>
</slide>
\noindent
As indicated in section 4, an environment meeting these requirements
may be used to create agent-based multi-user virtual environments.
We have a definte preference for a logic-based declarative language
that provides direct support for intelligentagents.
As we have shown in [_Community], both autonoums agents
and shared objectsmay be realized using agent-technology.
The platform must, furthermore,
provide sufficient networking capabilities,
so that changes to shared  objects in a world can be propagated 
among multiple users.
As a remark, in our platform we have used the X3D/VRML
External Authoring Interface (EAI) to control the 3D
environments programmatically.
<lp>
:@ scripting behavior  
Now, although a platform as described above offers
powerful computational capabilities, this is clearly
not enough to create embodied conversational agents with
a rich repertoire of gestures.
On top of this platform, we need a suitably expressive
scripting language for defining gestures and driving the behavior
of our humanoid agent avatars.
<lp>
In section 4.3, an outline is given of a gesture
scripting language for humanoids based on the H-Anim 1.1 standard.
The design of the scripting language was motivated by
the requirements listed below.

<slide id=ar-principle>
<c>
/// <em>scripting behavior</em>
<p>
</c>
<small>
<ul>
<li><em>convenience</em> -- for non-professional authors </li>
<li><em>compositional semantics</em> -- combining operations </li>
<li><em>re-definability</em> -- for high-level specification of actions </li>
<li><em>parametrization</em> -- for the adaptation of actions </li>
<li><em>interaction</em> -- with a (virtual) environment </li>
</ul>
</small>
<navigate/>
</slide>
\noindent
As described in section 4.3,
we developed a scripting language STEP
that, to our belief, meets these requirements.
STEP is based on dynamic logic and allows
for arbitrary abstractions using the primitives and
composition operators provided by our logic.

@* The DLP+X3D platform

In [_Community], we have described a platform for virtual
environments based on agent technology.
In effect, our platform is the result of merging VRML
with the distributed logic programming language DLP,
using the VRML External Authoring Interface.
This approach allows for a clear separation of concerns,
modeling 3D content on the one hand and determining the
dynamic behavior on the other hand.
As a remark, recently we have adopted [_X3D] as our 3D format.
The VRML profile of X3D is an XML encoding of VRML97.
<lp>
The language DLP is a distributed object-oriented extension
of Prolog, [_DLP].
It supports multiple inheritance, non-logical instance
variables and multi-threaded objects (to allow for distributed
backtracking).
Object methods arecollections of clauses.
Method invocation is dealt with as communication by rendez-vous,
for which synchronizationconditions may be specified in
so-called <em>accept</em> statements.
The current implementation of DLP is built on top of Java.
<lp>
To effect an interaction between the 3D content
and the behavioral component written in DLP,
we need to deal with two issues:

<slide id=ar-control>
<c>
/// DLP+X3D
</c><p>
<small>
<ul size=1.5 bullet=red>
<li> control points: <em>get/set</em> -- position, rotation, viewpoint </li>
<li> <em>event-handling</em> -- asynchronous accept</li>
</ul>
<navigate/>
</slide>
\noindent
We will explain each of these issues separately below.
In addition, we will indicate how multi-user environments
may be realized with our technology.

:@ control points 
The control points are actually nodes in the VRML scenegraph
thatact as handles which may be used to manipulate the scenegraph.
In effect, these handles are exactly the nodes that may act as the
source or target of event-routing in the 3D scene.
As an example, look at the code fragment below,
which gives a DLP rule to determine whether a soccer player
must shoot:

<slide id=ar-dlp-attr>
<p>
<small>
<display font=tt>
findHowToReact(Agent,Ball,Goal,shooting) :-
   get(Agent,position,sfvec3f(X,Y,Z)),
   get(Ball,position,sfvec3f(Xb,Yb,Zb)),
   get(Goal,position,sfvec3f(Xg,Yg,Zg)),
   distance(sfvec3f(X,Y,Z),sfvec3f(Xb,Yb,Zb),DistB),
   distance(sfvec3f(X,Y,Z),sfvec3f(Xg,Yg,Zg),DistG),
   DistB =< kickableDistance,
   DistG =< kickableGoalDistance.
</display>
</small>
<navigate/>
</slide>
\noindent
This rule will only succeed when the actual distance
of the player to the goal and to the ball
satisfies particular conditions.
<lp>
In addition to observing the state of the 3D scene
using the <em>get</em>predicate,
changes to the scene may be effected using the <em>set</em>
predicate.

:@ event handling 
Our approach also allows for changes in the scene that
are not a  direct result of setting attributes
from the logic component.
Therefore we need some way to intercept events.
<lp>
In the example below, we have specified an observer object
that has knowledgeof, that is inherits from, a dialog object.
<lp>
<slide id=ar-dlp-observer>
<c>
/// <em>observer</em>
</c><p>
<small>
<display font=tt>
:- object observer : [dialog].
var slide = anonymous, level = 0, projector = nil.

observer(X) :- 
   projector := X,
   repeat,
     accept( 
       id(_V,_T,_N), level(_V,_T,_N), update(_V,_T,_N), touched(_V,_T,_N)
     ), fail.

id(V,_T,_N) :- format('id ~w~n',[V]), !, slide := V.
level(V,_T,_N) :- level := V.
touched(V,T,_N) :- projector$->$touched(V,T,_N).
update(V,T,_N) :- !, act(V,slide,level).
:- end_object observer.
</display>
</small>
</slide>
\noindent
The constructor sets the non-logical variable <em>projector</em>
and enters a so-called failure-driven loop to accept
any of the incoming events for respectively <em>id</em>,
<em>level</em>, <em>update</em> and <em>touched</em>.
To receive events, the <em>observer</em> object
must be installed as the listener for these particular events.
<lp>
The events come from the 3D scene.
For example, the <em>touched</em> event results from mouse clicks
on a particular object in the scene.
On accepting an event, the corresponding method or clause
is activated, resulting in either changing the value
of a non-logical instance variable,
invoking a method,
or delegating the call to another object.


:@ multi-user virtual environments 
The DLP platform offers direct support for autonomous
agents, as for example the players in the soccer game
described in section 4.1.
However, our agent technology may also be used to realize
multi-user virtual environments.
In particular, we distinguish between the following
types of agents:

<slide id=ar-agents-mu>
<c>
/// agents
</c><p>
<small>
<ul bullet=green>
<li><em><red>object agents</red></em> -- control single shared objects
(<em><text text=blue>pilot at server, drone at client</text></em>)</li>
<li><em><red>user agent</red></em> -- controls users' avatar 
(<em><text text=blue>pilot at user side, drone at server or clients</text></em>)
</li>
<li><em><red>autonomous agents</red></em>  -- like football player, with own avatar 
(<em><text text=blue>pilot at server, drone at clients </text></em>)</li>
</ul>
</small>
<phrase level=0 right="That is~na lot~nof agents!" style="[plain,balloon=0]"/>
<phrase level=0 left="It sure is." style="[plain,balloon=0]"/>
<navigate/>
</slide>
\noindent
The terminology, <em>pilot</em> and <em>drone</em>,
stems from the <em>Living Worlds</em> proposal,
to indicate the difference between the original object
(<em>pilot</em>) and the objects that replicate that object
(<em>drones</em>).
It extends the taxonomy of agents introduced in [_Taxonomy]
to allow for shared objects.


@* Case studies

To illustrate the potential of our DLP+X3D platform,
we will briefly sketch three case studies, respectively
a multi-user soccer game with autonomous player agents,
the use of dialogs in VR presentations,
and a scripting language for specifying
gestures and movements for humanoids.

@+ Multi-user soccer

The various aspects of our multi-user soccer game are reported
in more detail in [_Community].
We chose the soccer game as a demonstrator or
target application because it provided us with a number
of challenges, as indicated below.

<slide id=ar-mu-socc>
<c>
/// <em>multi-user soccer game</em>
</c><p>
<small>
<ul bullet=red>
<li><em><blue>multiple (human) users</font></em> -- may join during the game </li>
<li><em><blue>multiple agents</font></em> -- to participate in the game (e.g. goalkeeper)</li>
<li><em><blue>reactivity</font></em> -- players (users and agents) have to react quickly</li>
<li><em><blue>cooperation/competition</font></em> -- requires intelligent communication</li>
<li><em><blue>dynamic behavior</font></em> -- sufficiently complex (dynamic) 3D scenes</li>
</ul>
</small>
<navigate/>
</slide>
\noindent
The soccer player agents each have a simple cognitive loop
based on the extended BDI model described in [_Avatars], which may be summarized
as <em>sense</em>, <em>think</em>, <em>act</em>.
<lp>
Furthermore, to allow multiple users to join the
game, taking the place of autonomous agent players if necessary,
we designed a special purpose <em>Agent Communication Language</em>
(ACL) to deal with the communication necessary to keep
the world updated, as previously outlined in the discussion on
multi-user virtual environments.
The message format is, schematically: <em>Action, Type, Parameters</em>.

<c>
\vspace{-3cm}
<slide id=ar-soccer-img>
<image name=@archive/wasp/wasp-soccer1>
</slide>
\vspace{-3cm}
\noindent          
</c>

<c>
<slide id=ar-soccer-protocol>
<p><p>
<center>
<text text=blue x=5 size=1.5> [ Act, Type, Parameters ] </text>
</center>
<navigate>
</slide>
</c>

<slide id=ar-proto-messages>
<c>
/// <em>distributed game protocol</em>
</c><p>
<small>
<ul bullet=red size=0.8>
<li><em>register game</em>: [ register, game_name, from(Host) ]</li>
<li><em>tell new player</em>: [ tell, new_player, user(Host,Name) ]</li>
<li><em>tell kick ball</em>: [ tell, kick_ball, [ user(Host,Name), force(X,Y,Z) ] ]</li>
<li><em>ask game score</em>: [ ask, game_score, user(Host,Name) ]</li>
</ul>
</small>
<navigate/>
</slide>
\noindent
Other message types include:
<slide id=ar-proto-x>
<p><blue><set size=0.7>
<quote><small><em>
register accept, register wait, tell position, tell rotation, tell game score, reply game score, unregister game, reply unregister game, player gone ...
</em></small></quote>
</font><set size=1.0>
<navigate/>
</slide>
\noindent
It is interesting to note that we may significantly optimize
on the communication load if we introduce additional
compound messages such as <em>run &amp; trace ball</em>,
to replace a sequence of primitive messages.

@+ Dialogs in Virtual Environments

Desktop VR is an excellent medium for presenting information,
for example in class, in particular when rich media
or 3D content is involved.
At VU, we have been using <em>presentational VR</em>
for quite some time, and recently we included dialogs
using balloons (and possibly avatars) to display the text
commenting on an a particular presentation.
Each presentation is organized as a sequence of slides,
and dependent on the slides (or level within the slide)
a dialog may be selected and displayed.
See the <em>observer</em> fragment in section 3.
<lp>
Our annotation for dialog text in slides looks as follows:

<slide id=ar-dialog-phr>
<c>
/// <em>dialog</em>
</c><p>
<small>
<display font=tt>
&lt;phrase right="how~nare~nyou"&gt;
%%&lt;phrase left="fine~nthank~nyou"/&gt;
%%&lt;phrase right="tell~nme~n..."/&gt;
...
%%&lt;phrase left="yes~neh~n..."/&gt;
%%&lt;phrase right="what do~nyou think~nof"/&gt;
%%&lt;phrase left="what?~n..."/&gt;
%%&lt;phrase right="the opportunities~n~n!"/&gt;
%%&lt;phrase left="what do~nyou mean?"/&gt;
%%&lt;phrase right="of studying~n..."/&gt;
%%&lt;phrase left="studying what~n...?"/&gt;
%%&lt;phrase right="please~nlet me know~n..."/&gt;
%%&lt;phrase left="yes eh,~nI think ..."/&gt;
%%&lt;phrase right="now,~nwhat do you~nthink?"/&gt;
%%&lt;phrase left="I think~nthat~nstudying~n..."/&gt;
%%&lt;phrase right="what~nabout~nstudying?"/&gt;
%%&lt;phrase left="studying~ninvolves~ndiscipline"/&gt;  
%%&lt;phrase right="yes eh~n?"/&gt;
%%&lt;phrase left="and~nmotivation~n..."/&gt;
%%&lt;phrase right="It is like~nliving!"/&gt;
%%&lt;phrase left="indeed~n!"/&gt;
%%&lt;phrase right="But ~nI am~nnot alive"/&gt;
&lt;phrase left="So,~nwhat~nare you?"/&gt;
&lt;phrase right="an ~nagent" style="[a(e)=1]"/&gt;
&lt;phrase left="I always~nwanted to be~nan agent" style="[a(e)=1]"/&gt;
</display>
<navigate/>
</slide>

<c>
\vspace{-4cm}
<slide type=fig xcaption="Dialogs in VR" id=ar-dial-img>
<image src=local/screens/vu-6.jpg pdf=local/screens/vu-6.pdf>
</slide>
\vspace{-4cm}
\noindent          
</c>
\noindent
Furthermore, there are a number of style parameters
to be dealt with to decide for example
whether the avatars or persona are visible, where to place
the dialogs balloons on the display, as well as the color
and transparancy of the balloons.
To this end, we have included a <em>style</em> attribute
in the <em>phrase</em> tag, to allow for setting
any of the style parameters:

<slide id=ar-dialog-style>
<c>
/// <em>style parameters</em>
</c><p>
<small>
<display font=tt>
&lt;phrase right="red" style="[p=(0.5,0,0),persona=0,balloon=0]"/&gt;
%%&lt;phrase left="cutie" style="[p=(-0.5,0,0),persona=0,balloon=0]"/&gt;
%%&lt;gesture right=1 style=default/&gt;
&lt;gesture left=1 style=default/&gt;
</display>
</small>
<navigate/>
</slide>
\noindent
Apart from phrases, we also allow for gestures, taken from
the built-in repertoire of the avatars.
Both phrases and gestures are compiled into DLP code
and loaded when the annotated version of the presentation VR
is started.
Currently, we do not allow for any variations within the
text provided in a phrase.

<c>
:@ design issues

<slide id=ar-level>
/// <a href=levels.html><red><em>levels of complexity</em></font></a>
<p>
<small>
<ul>
<li> level 0: <em>basic material</em> --
<a href=dialog-0-text.html>dialog text</a>,
<a href=dialog-0-media.html>media object</a></li>
<li> level 1: <em>combined</em> --
<a href=dialog-1-a.html>media object + dialog</a></li>
<li>level 2: <em>with avatars</em> --
<a href=dialog-2-a.html>media object + dialog + avatars</a></li>
<li>level 3: <em>with attributes</em> --
%%<a href=dialog-3-a.html>dialog + avatars + objects</a></li>
<a href=dialog-3-b.html>media object + dialog + avatars + objects</a></li>
<li>level 4: <em>with context</em> --
%%<a href=dialog-4-a.html>dialog + avatars + context</a></li>
<a href=dialog-4-b.html>media object + dialog + avatars + context</a></li>
%%<a href=dialog-4-c.html>media object + dialog + avatars + context + attributes</a></li>
</ul>
</small>
<navigate/>
</slide>
</c>

@+ STEP -- a scripting language for embodied agents

Given the use of humanoid avatars to comment on the
contents of a presentation,we may wish to enrich
the repertoire of gestures and movements to be able,
for example, to include gestural comments or even
instructions by gestures.
<lp>
Recently, we have started working on a scripting language for
humanoids based on dynamic logic, [_STEP].
The STEP scripting language consists of basic actions,
composite operators and interaction operators (to deal
with the environment in which the movements and actions take place).
<lp>
The basic actions of STEP consist of:

<slide id=ar-step-actions>
<c>
/// <em>basic actions</em>
</c><p>
<small>
<ul>
<li> <em>move</em> -- <tt>move(Agent,BodyPart,Direction,Duration)</tt> </li>
<li> <em>turn</em> -- <tt>turn(Agent,BodyPart,Direction,Duration)</tt> </li>
</ul>
</small>
<navigate/>
</slide>
\noindent
These basic actions are translated into operations
on the control points as specified by the H-Anim 1.1 standard, [_H-Anim].
<lp>
As composite operators we provide sequential and parallel
composition, as well as <em>choice</em>and <em>repeat</em>.

<slide id=ar-step-composite>
<c>
/// <em>composite operators</em>
</c><p>
<small>
<ul>
<li> <em>sequence</em> -- <tt>[Action$_1$,...,Action$_{n}$]</tt> </li>
<li> <em>parallel</em> -- <tt>parallel([Action$_1$,...,Action$_{n}$])</tt></li>
<li> <em>choice</em> -- <tt>disjunct([Action$_1$,...,Action$_{n}$])</tt></li>
<li> <em>repeat</em> -- <tt>repeat(Action,k)</tt></li>
</ul>
</small>
<navigate/>
</slide>
\noindent
Each action is defined using the <em>script</em>,
by specifying an action list containing
the (possibly compound) actions of which that particular
action consists.
As an example, look at the definition of <em>walking</em> below.


<slide id=ar-step-ex>
<c>
/// <em>example</em>
</c><p>
<small>
<display font=tt>
script(walk(Agent), ActionList) :-
ActionList = [
  parallel([turn(Agent,r_shoulder,back_down2,fast),
 	turn(Agent,r_hip,front_down2,fast),
 	turn(Agent,l_shoulder,front_down2,fast),
 	turn(Agent,l_hip,back_down2,fast)]),
  parallel([turn(Agent,l_shoulder,back_down2,fast),
 	turn(Agent,l_hip,front_down2,fast),
 	turn(Agent,r_shoulder,front_down2,fast),
 	turn(Agent,r_hip,back_down2,fast)])
], !.
</display>
</small>
<navigate/>
</slide>
\noindent
Notice that the <em>Agent</em> that is to perform the
movement is given as a parameter.
(Identifiers starting with a capital act as a logical parameter
or variable in Prolog and DLP.)
<lp>
Interaction operators are needed to conditionally
perform actions or to effect changes within the environment
by executing some command.
Our interaction operators include:

<slide id=ar-step-interacton>
<c>
/// <em>interaction</em>
</c><p>
<small>
<ul>
<li> <em>test</em> -- <tt>test(State)</tt> </li>
<li> <em>execution</em> -- <tt>do(State)</tt> </li>
<li> <em>conditional</em> -- <tt>if_ten_else(State,Action$_1$,Action$_2$)</tt>  </li>
<li> <em>until</em> -- <tt>until(Action,State)</tt></li>
</ul>
</small>
<navigate/>
</slide>
\noindent
Potentially, an action may result in many parallel
activities.
To control the number of threads used for an action,
we have created a scheduler that assigns
activities to a thread from a thread pool consisting
of a fixed number of threads.
<lp>
As a demonstrator for STEP, we envisage to create
an instructional VR for <em>Tai Chi</em>,
the Chinese art of movement.

:@ XML encoding 
Since we do not wish to force the average user
to learn DLP to be able to define scripts in STEP,
we are also developing X-STEP, an XML encoding for STEP.
As an example, look at the X-STEP specification
of the <em>walk</em> action.
<slide id=ar-step-ex>
<c>
/// X-STEP 
</c>
<small>
<display>
&lt;action type=walk param=[Agent]&gt;
&lt;seq&gt;
 &lt;par speed=fast&gt;
  &lt;gesture type=turn actor=Agent part=r_shoulder dir=back_down2/&gt;
  ...
%%  &lt;gesture type=turn actor=Agent part=r_hip dir=front_down2/&gt;
%%  &lt;gesture type=turn actor=Agent part=l_shoulder dir=front_down2/&gt;
%%  &lt;gesture type=turn actor=Agent part=l_hip dir=back_down2/&gt;
 &lt;/par&gt;
 &lt;par speed=fast&gt;
%%  &lt;gesture type=turn actor=Agent part=l_shoulder dir=back_down2/&gt;
%%  &lt;gesture type=turn actor=Agent part=l_hip dir=front_down2/&gt;
%%  &lt;gesture type=turn actor=Agent part=r_shoulder dir=front_down2/&gt;
    ...
  &lt;gesture type=turn actor=Agent part=r_hip dir=back_down2/&gt;
 &lt;/par&gt;
&lt;/seq&gt;
&lt;/action&gt;
</display>
</small>
<navigate/>
</slide>
\noindent
Similar as with the specification of dialog phrases,
such a specification is translated into the corresponding
DLP code, which is loaded with the scene it belongs to.

@* Evaluation criteria

The primary criterium against which to evaluate
applications that involve embodied conversational
agents is whether the application becomes more effective
by using such agents, effective in terms of communication
with the user.
Evidently, for the <em>Signing Avatar</em> applications this
seems to be quite obvious.
For other applications, for example negotiation in e-commerce,
this question might be more difficult to answer.
<lp>
As concerns the embedding of conversationl agents in VR,
we might make a distinction between <em>presentational VR</em>,
<em>instructional VR</em> and <em>educational VR</em>.
An examle of educational VR is described in [_EducationalVR].
No mention of agents was made in the latter reference though.
In instructional VR, explaining for example the use of a machine,
the appearance of a conversational agent seems to be quite natural.
In presentational VR, however,
the appearance of such agents might be considered no more 
than a gimmick.
<lp>
Considering the use of agents in applications in general,
we must make a distinction between <em>information agents</em>,
<em>presentation agents</em> and <em>conversational agents</em>.
Although the boundaries between these categories are not clearcut,
there seems to be an increasing degree of interactivity with the user.
<lp>
From a system perspective, we might be interested in what range of 
agent categories the system covers.
Does it provide support for managing information and possibly
information retrieval?
Another issue in this regard could be whether the system
is built around open standards, such as XML and X3D, to allow for
the incorporation of a variety of content.
<lp>
Last but not least, froma user perspective, what seems
to matter most is the naturalness of the (conversational) agents.
This is determined by the graphical quality, as well as
contextual parameters, that is how well the agentis embedded
in its environment.
More important even are emotive parameters,
that is the mood and style (in gestures and possibly speech)
with which the agents manifestthemselves.
In other words, the properties that determine whether
an agent is (really) convincing.

@* Related work

There is an enormous amount of research dealing with
virtual environments thatare in one way or another
inhabited by embodied agents.
By way of comparison, we will discuss a limited number of
related research projects.
<lp>
As systems that have a comparable scope we may mention
[_Environments] and [_DIVE], that both have a client-server
architecture for realizing virtual environments.
Our DLP+X3D platform distinguisghes itself from these
by providing a uniform programmatic interface, uniform 
in the sense of being based on DLP throughout.
<lp>
The [_Parlevink] gropu at the Dutch University of Twente
has done active research in applications of virtual
environments with agents.
Their focus is, however, more on language processing,
whereas our focus may be characterized as 
providing innovative technology.
<lp>
Both [_Jinni] and [_Scripts] deal with incorporating
logic programming within VRML-based scenes, the former
using the External Authoring Interface, and the latter
inline logic scripts.
With regard to [_Jinni], we claim that our programming
platform is much more powerful,
being object-oriented and not restricted to
single user systems.
<lp>
Our scripting language may be compared to
the scripting facilities offered by [_Alice],
which are built on top of Python.
Also, <em>Signing Avatars</em> has a powerful scripting language.
However,we wish to state that our scripting language is
based on dynamic logic, and has powerful abstraction
capabilities and support for parallelism.
<lp>
Finally, we seem to share a number of interests
with the [_VHML] community,
which is developing a suite of markup languages
for expressing humanoid behavior.
We see this activity as complementary to ours,
since our research proceeds from
technical feasibility,
that is how we can capture the semantics of
humanoid gestures and movements within
our dynamic logic,
which is implemented on top of DLP.


%%<ul>
%%<li> VHML -- Virtual Human Markup Language </li> 
%%</ul>



@* Conclusions and future research

<c>
<slide color=yellow id=conc-2>
<effect type=style sky="0 1 1" drop=0 logo="DLP+X3D"/>
<small>
<h4 align=right text=orange>PRO</h4>
<ul bullet=gold>
<li><em>high level platform </em>-- flexible and powerful </li>
<li><em>clean separation of concerns</em> -- modelling and programming</li>
</ul>
<h4 align=right text=orange>CON</h4>
<ul bullet=gold>
<li><em>added complexity</em> -- due to hybrid platform </li>
<li><em>performance penalty</em> -- due to EAI communication</li>
</ul>
</small>
<navigate/>
</slide>
</c>

In summary, we may state that our DLP+X3D platform is a powerful,
flexible and high-level platform for developing
VR applications with embodied agents.
It offers a clean separation of modeling and programming concerns.
On the negative side, we should mention that
this separation may also make development more complex
and, of course, that there is a (small) performance penalty
due to the overhead incurred
by using the External Authoring Interface.
<lp>
Where our system is currently lacking, clearly,
%%is the support it offers in terms of adequate
is adequate computational models underlying humanoid behavior,
including gestures, speech and emotive characteristics.
The VHML effort seems to have a rich offering that we need
to digest in order to improve our system in this respect.
<lp>
Our choice to adopt open standards, such as XML-based X3D,
seems to be benificial, in that it allows us to profit
from the work that is being done in other communities,
so that we can enrich our platform with the functionality
needed to create convincing embodied agents
in a meaningful context.

<include file=@biblio/refs>
</document>
